# Examples 9.1-9.3 of
# Mehttalo, L. and Lappi, J. 2020. Biometry for forestry and environmental data: With examples in R. CRC Press. 
# 
# Author: Lauri Mehttalo
###############################################################################

##### Example 9.1
library(lmfor)
data(stumplift)

# Fit bivariate models for productivity and processing time 
# Start with univariate models

modT<-gls(Time~Machine*Diameter,data=stumplift)
modP<-gls(Productivity~Machine*Diameter,data=stumplift)
cor(resid(modP),resid(modT))

modTplusP<-gls(I(Productivity+Time)~Machine*Diameter,data=stumplift)
covTP<-(modTplusP$sigma^2-modT$sigma^2-modP$sigma^2)/2
covTP/(modT$sigma*modP$sigma)

windows()
plot(resid(modP),resid(modT))

# estimation using package systemfit
install.packages("systemfit")
library(systemfit)
eqT<-Time~Machine*Diameter
eqP<-Productivity~Machine*Diameter
system<-list(Time=eqT,Productivity=eqP)
modSUR<-systemfit(system,"SUR",data=stumplift,maxit=10)
summary(modSUR)

summary(modT)

# Observation id is needed to tell R which rows are associated with
# the same sampling unit
stumplift$obs<-1:dim(stumplift)[1]
stumplift$M1<-as.numeric(stumplift$Machine==2)
# Generate data sets for both responses and pool them 
dat1<-data.frame(response=1,obs=stumplift$obs,y=stumplift$Time,
                 model.matrix(Time~Machine*Diameter,data=stumplift),0,0,0,0,0,0)
dat2<-data.frame(response=2,obs=stumplift$obs,y=stumplift$Productivity,0,0,0,0,0,0,
                 model.matrix(Productivity~Machine*Diameter,data=stumplift))
names(dat1)[-(1:3)]<-names(dat2)[-(1:3)]<-c("IT","M2T","M3T","DT","M2DT","M3DT",
                                            "IP","M2P","M3P","DP","M2DP","M3DP")
mvdat<-rbind(dat1,dat2)
# Show the lines for first five observations and the last
mvdat[c(1:5,485:490,970),]

library(nlme)
mvmod1<-gls(y~-1+IT+M2T+M3T+DT+M2DT+M3DT+IP+M2P+M3P+DP+M2DP+M3DP,
            weights=varIdent(form=~1|response),
            corr=corSymm(form=~1|obs),
            data=mvdat)
summary(mvmod1)

windows()
plot(fitted(mvmod1)[mvdat$response==1],resid(mvmod1,type="p")[mvdat$response==1])
plot(fitted(mvmod1)[mvdat$response==2],resid(mvmod1,type="p")[mvdat$response==2])

# Fit univariate models using varPower
modt<-gls(Time~Machine*Diameter,
          weights=varPower(),
          data=stumplift)

modp<-gls(Productivity~Machine*Diameter,
          weights=varPower(),
          data=stumplift)

# Save the estimated response-specific variance function power to mvdat
mvdat$delta<-attributes(modt$apVar)$Pars[1]
mvdat$delta[mvdat$response==2]<-attributes(modp$apVar)$Pars[1]
mvdat$z<-fitted(mvmod1)^(2*mvdat$delta)

mvmod2<-update(mvmod1,weights=varComb(varIdent(form=~1|response),varFixed(~z)))
	
print(mvmod2$sigma)
for (i in 1:10) {
	mvdat$z<-fitted(mvmod2)^(2*mvdat$delta)	
	mvmod2<-update(mvmod2)
	print(mvmod2$sigma)
}

windows(width=7,height=3)
par(mfcol=c(1,2),mai=c(0.6,0.6,0.1,0.1),mgp=c(2,0.7,0),cex=0.9)
plot(fitted(mvmod2)[mvdat$response==1],resid(mvmod2,type="p")[mvdat$response==1],
     xlab="Predicted processing time, s",ylab="Pearson residual",ylim=c(-4,6))
mywhiskers(fitted(mvmod2)[mvdat$response==1],
           resid(mvmod2,type="p")[mvdat$response==1],add=TRUE,se=FALSE)
plot(fitted(mvmod2)[mvdat$response==2],resid(mvmod2,type="p")[mvdat$response==2],
     xlab=expression("Predicted productivity, "*m^3/h),ylab="Pearson residual",ylim=c(-4,6))
mywhiskers(fitted(mvmod2)[mvdat$response==2],
           resid(mvmod2,type="p")[mvdat$response==2],add=TRUE,se=FALSE)

summary(mvmod2)
#####


##### Example 9.2, Inference
anova(mvmod2,Terms=c("M2T","M3T","M2DT","M3DT","M2P","M3P","M2DP","M3DP"))
#####


##### Example 9.3, Prediction
# Predict using fixed part only
beta<-coef(mvmod2)
x1<-c(1,0,0,40,0,0,rep(0,6))
mu1<-x1%*%beta
y1pred0<-mu1                               # 56.74
V1<-sigmat^2*mu1^(2*deltat)                # 429.40
vary1pred0<-t(x1)%*%mvmod2$varBeta%*%x1+V1 # 433.88
# Prediction interval
as.numeric(y1pred0)+qnorm(c(0.025,0.975))*as.numeric(sqrt(vary1pred0))

# Prediction of processing time for a stump of diameter 40 cm processed using machine 1
# Extract the variance-covariance parameters from the fitted models
phi<-2*plogis(attributes(mvmod2$apVar)$Pars[1])-1
sigmat<-mvmod2$sigma
sigmap<-exp(attributes(mvmod2$apVar)$Pars)[2]*sigmat
deltat<-attributes(modt$apVar)$Pars[1]
deltap<-attributes(modp$apVar)$Pars[1]

# Prediction using known productivity of 17.27
h2<-17.27
x2<-c(rep(0,6),1,0,0,40,0,0)
mu2<-x2%*%beta                            # 12.73
V2<-sigmap^2*mu2^(2*deltap)               # 23.68
V12<-phi*sqrt(V1*V2)                      # -81.75
y1pred1<-mu1+V12%*%solve(V2)*(h2-mu2)     # 41.07
vary1pred1<-t(x1)%*%mvmod2$varBeta%*%x1+V1-V12%*%solve(V2)%*%t(V12) # 151.63
as.numeric(y1pred1)+qnorm(c(0.025,0.975))*as.numeric(sqrt(vary1pred1))
#####


